precision mediump float;

uniform sampler2D textureUnit0;
uniform sampler2D textureUnit1;
uniform sampler2D textureUnit2;
uniform sampler2D textureUnit3;

uniform int textureCount;
uniform int blendingMode[4];

uniform int grayMode;

varying vec2 texCoord0;
varying vec2 texCoord1;
varying vec4 vertexColor;
varying float fogWeight;
varying vec3 fogVertexColor;

const vec4 WHITE = vec4(1,1,1,1);

void main() {
	vec4 col = texture2D(textureUnit0, texCoord0) * vertexColor;

	if(grayMode == 1) {
		float grayColor = (col.x + col.y + col.z) * 0.333;
		col.x = grayColor;
		col.y = grayColor;
		col.z = grayColor;
	} else if(grayMode == 2) {
		col.x = col.x * 2.0;
		col.y = col.y * 0.5;
		col.z = col.z * 0.5;
	} else if(grayMode == 3) {
		col.x = col.x * 0.5;
		col.y = col.y * 0.5;
		col.z = col.z * 1.0;
	}
	
	
	if (textureCount>1) {
	
		// Can't index texture samplers and switch doesn't seem to compile(?)...end result:
	
		int mode=blendingMode[1];
		vec2 texCo=texCoord1;
	
		if (mode==0) {
			// Modulate
			col *= texture2D(textureUnit1, texCo);
		} else if (mode==1) {
			// Add
			col += texture2D(textureUnit1, texCo);
		} else if (mode==3) {
			// Blend
			col *= (WHITE - texture2D(textureUnit1, texCo));
		} else if (mode==2) {
			// Replace
			col = texture2D(textureUnit1, texCo);
		} else if (mode==4) {
			// Decal
			vec4 col2=texture2D(textureUnit1, texCo);
			col = vec4(mix(col.rgb, col2.rgb, col2.a), col2.a); 
		}
	}
	
	if (fogWeight>-0.9) {
		col.xyz = (1.0-fogWeight) * col.xyz + fogVertexColor;
	}
	
	gl_FragColor=col;
}


