uniform mat4 modelViewMatrix;
uniform mat4 modelViewProjectionMatrix;
uniform mat4 textureMatrix;

uniform vec4 additionalColor;
uniform vec4 ambientColor;

uniform float alpha;
uniform float shininess;
uniform bool useColors;

uniform float fogStart;
uniform float fogEnd;
uniform vec3 fogColor;

uniform int lightCount;

uniform int screenEffect;

uniform vec4 lightmapParam; // XZhJ
uniform vec3 lightPositions[8]; //YJJ Attention: THINK TWICE before you use this!  Some mobile phone has problems with array, like K-Touch W96!
uniform vec3 diffuseColors[8];
uniform vec3 specularColors[8];
uniform float attenuation[8];

attribute vec4 position;
attribute vec3 normal;
attribute vec4 color;
attribute vec2 texture0;
attribute vec2 texture1;
attribute vec2 texture2;
attribute vec2 texture3;

varying vec2 texCoord0;
varying vec2 texCoord1;
varying vec4 vertexColor;
varying vec3 fogVertexColor;
varying float fogWeight;

const vec4 WHITE = vec4(1,1,1,1);
const float WAVE_SWING_1 = 30.0;
const float WAVE_SWING_2 = 5.0;

void main() {
	texCoord0 = (textureMatrix * vec4(texture0, 0, 1)).xy;	
	
	float U = (lightmapParam.x -position.x) * lightmapParam.z;
    float V = (lightmapParam.y - position.z) * lightmapParam.w;
	texCoord1 = vec2(U,V);

	vec4 vertexPos = modelViewMatrix * position;
	vertexColor = ambientColor + additionalColor;
	
	if (fogStart != -1.0) {
		fogWeight = clamp((-vertexPos.z - fogStart) / (fogEnd - fogStart), 0.0, 1.0);
		fogVertexColor = fogColor * fogWeight;
	} else {
		fogWeight = -1.0;
	}
	
	vertexColor=vec4(min(WHITE, vertexColor).xyz, alpha);
	
	if (useColors) {
		vertexColor *= color;
	}
	
	gl_Position = modelViewProjectionMatrix * position;
	
	if (screenEffect == 1) {
		float offset_1 = WAVE_SWING_1 * sin(gl_Position.z * 0.01);
		float offset_2 = WAVE_SWING_2 * cos(gl_Position.z);
		gl_Position.x += offset_1 + offset_2;
	}
}